<?php
namespace Teknisa\CMP\Services;

use \Zeedhi\Framework\DB\StoredProcedure\StoredProcedure;
use \Zeedhi\Framework\DB\StoredProcedure\Param;

class ContaPagarService {
    
    protected $entityManager;
    protected $connection;
    protected $environment;

    public function __construct($entityManager, $environment) {
        $this->entityManager = $entityManager;
        $this->connection = $entityManager->getConnection();
        $this->environment = $environment;
    }
    
    
    public function executaSql($sql, $params, $consulta = true) {
        try{
            $std = $this->connection->prepare($sql);
            $std->execute($params);
            
            if($consulta == true) {
                return $std->fetchAll();
            }
            
            return true;
        } catch (\Exception $e){
            throw $e;
        } 
    }
    
    
    public function retornaTipoOper($cdfildest) {
        
        $select = "SELECT CDTIPOOPER FROM PARCMPSUP WHERE CDFILIAL = :CDFILIAL AND NRORG = :NRORG";
        
        $params = [
            "CDFILIAL" => $cdfildest,
            "NRORG" => $this->environment->getNrOrg()
        ];
        
        $cdtipooper = self::executaSql($select, $params);
        
        $cdtipooper = count($cdtipooper) ? $cdtipooper[0]['CDTIPOOPER'] : false;
        
        return $cdtipooper ? $cdtipooper : '01';
        
    }
    
    
    public function retornaTrabVencimento($cdfilialvenc, $nrlanctonfvenc) {
        
        $select = "SELECT * FROM TRAB_VENCIMENTO WHERE CDFILIALVENC = :P_CDFILIALVENC AND NRLANCTONFVENC = :P_NRLANCTONFVENC AND NRORG = :NRORG";
        
        $params = [
            "P_CDFILIALVENC"    => $cdfilialvenc,
            "P_NRLANCTONFVENC"  => $nrlanctonfvenc,
            "NRORG"             => $this->environment->getNrOrg()
        ];
        
        $trabVencimento = self::executaSql($select, $params);
        
        // return count($trabVencimento) ? $trabVencimento : false;
        return $trabVencimento;
    }
    
    
    public function retornaGeraContaPagarFornecedor($cdfildest, $cdfillanc, $nrautoforn) {
        
        $select = "SELECT FORNECEDOR.IDGERCONTPGT FROM AUTOFORNE, FORNECEDOR WHERE AUTOFORNE.CDFILDEST = :CDFILDEST AND AUTOFORNE.CDFILLANC = :CDFILLANC AND AUTOFORNE.NRAUTOFORN = :NRAUTOFORN AND AUTOFORNE.NRORG = :NRORG AND AUTOFORNE.CDFORNECED = FORNECEDOR.CDFORNECED AND AUTOFORNE.NRORG = FORNECEDOR.NRORG";
        
        $params = [
            "CDFILDEST" => $cdfildest,
            "CDFILLANC" => $cdfillanc,
            "NRAUTOFORN" => $nrautoforn,
            "NRORG" => $this->environment->getNrOrg()
        ];
        
        $idgercontpgt = self::executaSql($select, $params);
        
        $idgercontpgt = count($idgercontpgt) ? $idgercontpgt[0]['IDGERCONTPGT'] : false;
        
        return $idgercontpgt == 'S' ? true : false;
        
    }
    
    
    public function retornaClassificacaoFinanceira($cdfildest, $cdfillanc, $nrautoforn) {
        
        $select = "SELECT TIPOCTPAGAR.CDCLASSFINA, TIPOCTPAGAR.CDTPCTPAGAR FROM TIPOCTPAGAR, AUTOFORNE WHERE AUTOFORNE.CDTPCTPAGAR = TIPOCTPAGAR.CDTPCTPAGAR AND AUTOFORNE.CDFILDEST = :CDFILDEST AND AUTOFORNE.CDFILLANC = :CDFILLANC AND AUTOFORNE.NRAUTOFORN = :NRAUTOFORN AND AUTOFORNE.NRORG = :NRORG";
        
        $params = [
            "CDFILDEST" => $cdfildest,
            "CDFILLANC" => $cdfillanc,
            "NRAUTOFORN" => $nrautoforn,
            "NRORG" => $this->environment->getNrOrg() 
        ];
        
        $classFinan = self::executaSql($select, $params);
        
        return count($classFinan) ? $classFinan[0] : false;
        
    }
    
    
    public function retornaClassificacaoTipoOperador($cdtipooper) {
        
        $select = "SELECT TIPOCTPAGAR.CDCLASSFINA, TIPOCTPAGAR.CDTPCTPAGAR FROM TIPOCTPAGAR, TIPOOPERAC WHERE TIPOOPERAC.IDENTRSAIDOP = 'E' AND TIPOOPERAC.CDTIPOOPER = :CDTIPOOPER AND TIPOOPERAC.NRORG = :NRORG AND TIPOOPERAC.CDTPCTPAGAR IS NOT NULL AND TIPOOPERAC.CDTPCTPAGAR = TIPOCTPAGAR.CDTPCTPAGAR";
            
        $params = [
            "CDTIPOOPER" => $cdtipooper,
            "NRORG" => $this->environment->getNrOrg()
        ];
        
        $classTpOper = self::executaSql($select, $params);
        
       return count($classTpOper) ? $classTpOper[0] : false;
        
    }
    
    
    public function retornaAutoforneContaPagar($cdfildest, $cdfillanc, $nrautoforn) {
        
        $select = "SELECT TOTITM.DTPROGENTAF, TOTITM.CDEMPRESA, TOTITM.CDFORNECED, TOTITM.VRPROD, ROUND(TOTITM.TOT + NVL(TOTIMP.TOT,0) + NVL(ENCACRES.TOTACRES,0) - NVL(ENCDESC.TOTDESC,0), 5) AS VRTOTAL, ROUND(TOTITM.TOT + NVL(TOTIMP.TOT,0) + NVL(ENCACRES.TOTACRES,0) - NVL(ENCDESC.TOTDESC,0) - NVL(IMPRETER.TOTIMPRET, 0), 5) AS VRTOTCTPAGAR, ROUND(TOTITM.VRPROD_REST + NVL(TOTIMP.TOT_REST,0) + DECODE(TOTITM.VRPROD - TOTITM.VRPROD_REST, 0, - NVL(TOTITM.VRDESC,0) + NVL(ENCACRES.TOTACRES,0) - NVL(ENCDESC.TOTDESC,0) - NVL(IMPRETER.TOTIMPRET, 0), 0), 5) AS VRTOTAL_REST FROM(SELECT ITEMAUTF.DTPROGENTAF, AUTOFORNE.CDEMPRESA, AUTOFORNE.CDFORNECED, SUM((ITEMAUTF.QTPROGENTAF * ITEMAUTF.VRPREUNIAF) * (NVL(ITEMAUTF.VRPERDESAF,0) / 100)) AS VRDESC, SUM(ITEMAUTF.QTPROGENTAF * ITEMAUTF.VRPREUNIAF) AS VRPROD, SUM((ITEMAUTF.QTPROGENTAF - NVL(ITEMAUTF.QTEFETENTRAF,0)) * ITEMAUTF.VRPREUNIAF) AS VRPROD_REST, SUM((ITEMAUTF.QTPROGENTAF * ITEMAUTF.VRPREUNIAF) - ((ITEMAUTF.QTPROGENTAF * ITEMAUTF.VRPREUNIAF) * NVL(ITEMAUTF.VRPERDESAF,0) / 100)) AS TOT FROM ITEMAUTF, AUTOFORNE WHERE AUTOFORNE.CDFILDEST = :CDFILDEST AND AUTOFORNE.CDFILLANC = :CDFILLANC AND AUTOFORNE.NRAUTOFORN = :NRAUTOFORN AND AUTOFORNE.NRORG = :NRORG AND AUTOFORNE.CDFILDEST = ITEMAUTF.CDFILDEST AND AUTOFORNE.CDFILLANC = ITEMAUTF.CDFILLANC AND AUTOFORNE.NRAUTOFORN = ITEMAUTF.NRAUTOFORN GROUP BY ITEMAUTF.DTPROGENTAF, AUTOFORNE.CDEMPRESA, AUTOFORNE.CDFORNECED) TOTITM, ( SELECT IMPITMAF.DTPROGENTAF, AUTOFORNE.CDEMPRESA, AUTOFORNE.CDFORNECED, SUM (IMPITMAF.VRADIMPAF * (ITEMAUTF.QTPROGENTAF - NVL(ITEMAUTF.QTEFETENTRAF,0))/ITEMAUTF.QTPROGENTAF) AS TOT_REST, SUM (IMPITMAF.VRADIMPAF) AS TOT FROM IMPITMAF, ITEMAUTF, AUTOFORNE WHERE AUTOFORNE.CDFILDEST = :CDFILDEST AND AUTOFORNE.CDFILLANC = :CDFILLANC AND AUTOFORNE.NRAUTOFORN = :NRAUTOFORN AND AUTOFORNE.NRORG = :NRORG AND AUTOFORNE.CDFILDEST = ITEMAUTF.CDFILDEST AND AUTOFORNE.CDFILLANC = ITEMAUTF.CDFILLANC AND AUTOFORNE.NRAUTOFORN = ITEMAUTF.NRAUTOFORN AND IMPITMAF.CDFILDEST = ITEMAUTF.CDFILDEST AND IMPITMAF.CDFILLANC = ITEMAUTF.CDFILLANC AND IMPITMAF.NRAUTOFORN = ITEMAUTF.NRAUTOFORN AND IMPITMAF.CDPRODUTO = ITEMAUTF.CDPRODUTO AND IMPITMAF.CDMARCA = ITEMAUTF.CDMARCA AND IMPITMAF.DTPROGENTAF = ITEMAUTF.DTPROGENTAF GROUP BY IMPITMAF.DTPROGENTAF, AUTOFORNE.CDEMPRESA, AUTOFORNE.CDFORNECED) TOTIMP, ( SELECT ENCARGOAF.DTENTRENCAF, AUTOFORNE.CDEMPRESA, AUTOFORNE.CDFORNECED, SUM(ENCARGOAF.VRENCARGOAF) AS TOTACRES FROM ENCARGOAF, ENCARGO, AUTOFORNE WHERE AUTOFORNE.CDFILDEST = :CDFILDEST AND AUTOFORNE.CDFILLANC = :CDFILLANC AND AUTOFORNE.NRAUTOFORN = :NRAUTOFORN AND AUTOFORNE.NRORG = :NRORG AND AUTOFORNE.CDFILDEST = ENCARGOAF.CDFILDEST AND AUTOFORNE.CDFILLANC = ENCARGOAF.CDFILLANC AND AUTOFORNE.NRAUTOFORN = ENCARGOAF.NRAUTOFORN AND ENCARGOAF.CDENCARGO = ENCARGO.CDENCARGO AND ENCARGO.IDENCARGO = 'A' GROUP BY ENCARGOAF.DTENTRENCAF, AUTOFORNE.CDEMPRESA, AUTOFORNE.CDFORNECED) ENCACRES, ( SELECT ENCARGOAF.DTENTRENCAF, AUTOFORNE.CDEMPRESA, AUTOFORNE.CDFORNECED, SUM(ENCARGOAF.VRENCARGOAF) AS TOTDESC FROM ENCARGOAF, ENCARGO, AUTOFORNE WHERE AUTOFORNE.CDFILDEST = :CDFILDEST AND AUTOFORNE.CDFILLANC = :CDFILLANC AND AUTOFORNE.NRAUTOFORN = :NRAUTOFORN AND AUTOFORNE.NRORG = :NRORG AND AUTOFORNE.CDFILDEST = ENCARGOAF.CDFILDEST AND AUTOFORNE.CDFILLANC = ENCARGOAF.CDFILLANC AND AUTOFORNE.NRAUTOFORN = ENCARGOAF.NRAUTOFORN AND ENCARGOAF.CDENCARGO = ENCARGO.CDENCARGO AND ENCARGO.IDENCARGO = 'D' GROUP BY ENCARGOAF.DTENTRENCAF, AUTOFORNE.CDEMPRESA, AUTOFORNE.CDFORNECED) ENCDESC, ( SELECT IMPITMAF.DTPROGENTAF, SUM(IMPITMAF.VRIMPITMAF) AS TOTIMPRET FROM IMPOSTO, IMPITMAF WHERE IMPITMAF.CDFILDEST = :CDFILDEST AND IMPITMAF.CDFILLANC = :CDFILLANC AND IMPITMAF.NRAUTOFORN = :NRAUTOFORN AND IMPITMAF.NRORG = :NRORG AND IMPOSTO.CDIMPOSTO = IMPITMAF.CDIMPOSTO AND IMPOSTO.IDTRATIMPO = 'R' GROUP BY IMPITMAF.DTPROGENTAF) IMPRETER WHERE TOTITM.DTPROGENTAF = TOTIMP.DTPROGENTAF(+) AND TOTITM.DTPROGENTAF = ENCACRES.DTENTRENCAF (+) AND TOTITM.DTPROGENTAF = ENCDESC.DTENTRENCAF (+) AND TOTITM.DTPROGENTAF = IMPRETER.DTPROGENTAF (+) AND (NOT EXISTS (SELECT CDFILIAL FROM NOTAFISCAL, ITEMAUTF WHERE (:CDFILDEST = ITEMAUTF.CDFILDEST ) AND (:CDFILLANC = ITEMAUTF.CDFILLANC ) AND (:NRAUTOFORN = ITEMAUTF.NRAUTOFORN ) AND (:NRORG = ITEMAUTF.NRORG ) AND (ITEMAUTF.CDFILDEST = NOTAFISCAL.CDFILIAL ) AND (ITEMAUTF.NRLANCTONF = NOTAFISCAL.NRLANCTONF) AND (TOTITM.DTPROGENTAF = NOTAFISCAL.DTENTRSAID) AND (NOTAFISCAL.IDSISTEMA <> 'CMP' )))";
    
        $params = [
            "CDFILDEST" => $cdfildest,
            "CDFILLANC" => $cdfillanc,
            "NRAUTOFORN" => $nrautoforn,
            "NRORG" => $this->environment->getNrOrg() 
        ];
        
        $autoForne = self::executaSql($select, $params);
        
        return count($autoForne) ? $autoForne : false;
        
    }
    
    
    public function deletaTrabVencimento($cdfilialvenc, $nrlanctonfvenc) {
        
        $delete = "DELETE FROM TRAB_VENCIMENTO WHERE CDFILIALVENC = :P_CDFILIALVENC AND NRLANCTONFVENC = :P_NRLANCTONFVENC AND NRORG = :NRORG";
        
        $params = [
            "P_CDFILIALVENC"    => $cdfilialvenc,
            "P_NRLANCTONFVENC"  => $nrlanctonfvenc,
            "NRORG"             => $this->environment->getNrOrg()
        ];
        
        self::executaSql($delete, $params, false);
        
        return true;
        
    }
    
    
    public function proceExcluiContaPagarPrevista($cdfildest, $cdfillanc, $nrautoforn) {
        //without commit in procedure
        $procedure = new StoredProcedure($this->connection, 'EXCLUI_CONTAPAGAR_PREVISTA');
        
        $procedure->addParam(new Param('P_CDFILDEST', Param::PARAM_INPUT, $cdfildest, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_CDFILLANC', Param::PARAM_INPUT, $cdfillanc, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_NRAUTOFORN', Param::PARAM_INPUT, $nrautoforn, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_RESULTADO', Param::PARAM_OUTPUT,  null, null, 20));
        
        $resultado = $procedure->execute();
        
        return $resultado;
        
    }
    
    
    public function proceGeraContaPagarPrevista($cdfildest, $cdfillanc, $nrautoforn) {
        //without commit in procedure
        $procedure = new StoredProcedure($this->connection, 'GERA_CONTAPAGAR_PREVISTA');
        
        $procedure->addParam(new Param('P_CDFILDEST', Param::PARAM_INPUT, $cdfildest, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_CDFILLANC', Param::PARAM_INPUT, $cdfillanc, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_NRAUTOFORN', Param::PARAM_INPUT, $nrautoforn, Param::PARAM_TYPE_STR));
        
        $procedure->execute();
        
    }
    
    
    public function proceCriaVencimento($params) {
        //without commit in procedure
        $procedure = new StoredProcedure($this->connection, 'CRIA_VENCIMENTO');
        
        $procedure->addParam(new Param('P_CDFILIAL',        Param::PARAM_INPUT, $params['CDFILDEST'],       Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_NRLANCTONF',      Param::PARAM_INPUT, $params['NRAUTOFORN'],      Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_IDENTRSAIDOP',    Param::PARAM_INPUT, 'A',                        Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_VRPAGTO',         Param::PARAM_INPUT, $params['VRTOTAL_REST'],    Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_DTEMISSAO',       Param::PARAM_INPUT, $params['DTPROGENTAF'],     Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_DTENTRSAID',      Param::PARAM_INPUT, $params['DTPROGENTAF'],     Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_CDCLIEFORN',      Param::PARAM_INPUT, NULL,                       Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_CDFILIEMIRECE',   Param::PARAM_INPUT, NULL,                       Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_ORIGDESTNF',      Param::PARAM_INPUT, NULL,                       Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_CDFILLANC',       Param::PARAM_INPUT, $params['CDFILLANC'],       Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_CDFILDEST',       Param::PARAM_INPUT, $params['CDFILDEST'],       Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_NRAUTOFORN',      Param::PARAM_INPUT, $params['NRAUTOFORN'],      Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_CDFILPEDIDO',     Param::PARAM_INPUT, NULL,                       Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_NRPEDIDO',        Param::PARAM_INPUT, NULL,                       Param::PARAM_TYPE_STR));
        
        $procedure->execute();
        
    }
    
    
    public function proceOrcamentoOk($params) {
        //without commit in procedure
        $procedure = new StoredProcedure($this->connection, 'ORCAMENTO_OK');
        
        $procedure->addParam(new Param('P_CDEMPRESA',       Param::PARAM_INPUT, $params['CDEMPRESA'], Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_CDFILIAL',        Param::PARAM_INPUT, $params['CDFILIAL'], Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_CDCLASSFINA',     Param::PARAM_INPUT, $params['CDCLASSFINA'], Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_DTLANCAMENTO',    Param::PARAM_INPUT, $params['DTLANCAMENTO'], Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_VRVALOR',         Param::PARAM_INPUT, $params['VRVALOR'], Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_RESULTADO',       Param::PARAM_OUTPUT,  null, null, 20));
        
        $resultado = $procedure->execute();
        
        return $resultado;
        
    }
    












    
    
    
}
   